// bclabelctl.cpp : Implementation of Cbclabel


#include "stdafx.h"
#include "Bclabel.h"
#include "bclabelctl.h"
//#include "bccon.h"
#include "3dtext.h"
#include "conv.h"
#include "gbord.h"
#include "trnclr.h"

//used for a default font in case of no container font
static const FONTDESC _fontdesc =
      {sizeof(FONTDESC), OLESTR("times new roman"), FONTSIZE( 14 ),
      FW_BOLD, ANSI_CHARSET, TRUE, FALSE, FALSE };

#include "bcmouseptr.h"
#include "bcalign.h"
#include "bcvalign.h"
#include "bcborder.h"
#include "bctruefalse.h"

CHAR szBackStyle[]= "0 - Transparent\0"
            "1 - Opaque\0"
            "\0";

CHAR szAlignmentImage[]= "0 - Left\0"
            "1 - Right\0"
            "\0";

CHAR szFlashType[]= "0 - None\0"
            "1 - Black\0"
            "2 - Background\0"
            "3 - Color\0"
            "\0";


/////////////////////////////////////////////////////////////////////////////
// Cbclabel
Cbclabel::Cbclabel()
{
	m_bWindowOnly = TRUE;
	m_alignment = 2;
	m_valignment = 0;
	m_backcolor = VBCOLOR_BTNFACE;
	m_backstyle = 0;
	m_bordercolor = VBCOLOR_WINDOWFRAME;
	m_borderstyle = 1;
	m_bordersize = 3;
	m_caption = _T("ATL Label Control") ;
	m_enabled = -1;
	m_flashcolor = 0xff;
	m_flashtype = 0;
	m_flashtime=750;
	m_forecolor = VBCOLOR_WINDOWTEXT;
	m_helpcontextid = 0;
	m_pmouseicon;
	m_mousepointer = 0;
	m_wordwrap = FALSE;
	m_polecontrolsite=NULL;
	m_overcontrol = FALSE;
	m_sButton = 1;
	m_lastflash=0;
	m_curflash=0;
}

Cbclabel::~Cbclabel()
{
	if ((m_overcontrol) || (m_flashtype))
		::KillTimer(m_hWnd,1);
}

STDMETHODIMP Cbclabel::GetEnumList(CALPOLESTR *pCaStringsOut,CADWORD *pCaCookiesOut, LPTSTR lpElements, short cOffset)
{
	//find the number of elements
	//walk lpElements until double 0
	//count 0's along the way
	LPTSTR	lpTemp = lpElements;
	short	cElems = 0;
	short	i;
	int cch = 5;
	LPOLESTR strTemp = NULL;

	do
	{
		lpTemp++;
		if (*lpTemp==0)
		{
			cElems++;
			lpTemp++;
		}
	}
	while (*lpTemp!=0);
	// allocate memory for string array structure
	pCaStringsOut->pElems = (LPOLESTR*)CoTaskMemAlloc(sizeof(LPOLESTR) * cElems);
	if (pCaStringsOut->pElems == NULL)
		return E_OUTOFMEMORY;
	// allocate memory for dword array structure
	pCaCookiesOut->pElems = (DWORD*)CoTaskMemAlloc(sizeof(DWORD*) * cElems);
	if (pCaCookiesOut->pElems == NULL)
	{
		CoTaskMemFree(pCaStringsOut->pElems);
		return E_OUTOFMEMORY;
	}
	pCaStringsOut->cElems = cElems;
	pCaCookiesOut->cElems = cElems;
	// fill dword array structure
	for (i=0;i<cElems;i++)
	{
		pCaCookiesOut->pElems[i] = i+cOffset;
	}
	// fill string array structure
	lpTemp = lpElements;

	i=0;
	do
	{
		cch = lstrlen(lpTemp) + 1;
		if ((strTemp = (LPOLESTR)CoTaskMemAlloc(cch * sizeof(OLECHAR))) != NULL)
			MultiByteToWideChar(CP_ACP, 0, lpTemp, -1, strTemp, cch);
		pCaStringsOut->pElems[i] = strTemp;
		i++;
		lpTemp += lstrlen(lpTemp) + 1;
	}
	while (*lpTemp!=0);
	return S_OK;
}

STDMETHODIMP Cbclabel::GetEnumString(	BSTR	*pBstr,
									LPTSTR	lpElements,
									short	svalue,
									short	sOffset)
{
	LPTSTR	lpTemp = lpElements;
	short	i = sOffset;
	//go though the array to the value
	do
	{
		if (i == svalue)
			break;
		lpTemp++;
		if (*lpTemp==0)
		{
			i++;
			lpTemp++;
		}
	}
	while (*lpTemp!=0);

	*pBstr = A2BSTR(lpTemp);
	return S_OK;
}

//IPerPropertyBrowsing
STDMETHODIMP Cbclabel::MapPropertyToPage (DISPID dispid, CLSID *pClsid) 
{ 
	HRESULT hr = IPerPropertyBrowsingImpl<Cbclabel>::MapPropertyToPage(dispid, pClsid); 
	if (SUCCEEDED(hr) && CLSID_NULL == *pClsid) 
		hr = PERPROP_E_NOPAGEAVAILABLE; 
	return hr; 
}


STDMETHODIMP Cbclabel::GetDisplayString(DISPID dispID,BSTR *pBstr)
{
	switch (dispID)
	{
		case DISPID_BC_ALIGNMENT: //Alignment
			return GetEnumString(pBstr, szAlignment, m_alignment,0);
		case DISPID_BC_VALIGNMENT: //VAlignment
			return GetEnumString(pBstr, szVAlignment, m_valignment,0);
		case DISPID_BC_BACKSTYLE: //BackStyle
			return GetEnumString(pBstr, szBackStyle, m_backstyle,0);
		case DISPID_BC_BORDERSTYLE: //BorderStyle
			return GetEnumString(pBstr, szBorderStyle, m_borderstyle,0);
		case DISPID_BC_FLASHTYPE: //FlashType
			return GetEnumString(pBstr, szFlashType, m_flashtype,0);
		case DISPID_BC_MOUSEPOINTER: //MousePointer
			return GetEnumString(pBstr, szMousePointer, m_mousepointer,0);
		
		case DISPID_BC_ENABLED: //Enabled
			return GetEnumString(pBstr, sztruefalse, m_enabled,-1);
		case DISPID_BC_WORDWRAP: //WordWrap
			return GetEnumString(pBstr, sztruefalse, m_wordwrap,-1);

		default:
			return E_NOTIMPL;
	}
	return S_OK;
}

STDMETHODIMP Cbclabel::GetPredefinedStrings(DISPID dispID, CALPOLESTR *pCaStringsOut,CADWORD *pCaCookiesOut)
{
	if (pCaStringsOut && pCaCookiesOut)
	{
		switch (dispID)
		{
		case DISPID_BC_ALIGNMENT: //Alignment
			return GetEnumList(pCaStringsOut, pCaCookiesOut, (LPTSTR) szAlignment, 0);
		case DISPID_BC_VALIGNMENT: //VAlignment
			return GetEnumList(pCaStringsOut, pCaCookiesOut, (LPTSTR) szVAlignment, 0);
		case DISPID_BC_BACKSTYLE: //BackStyle
			return GetEnumList(pCaStringsOut, pCaCookiesOut, (LPTSTR) szBackStyle, 0);
		case DISPID_BC_BORDERSTYLE: //BorderStyle
			return GetEnumList(pCaStringsOut, pCaCookiesOut, (LPTSTR) szBorderStyle, 0);
		case DISPID_BC_FLASHTYPE: //FlashType
			return GetEnumList(pCaStringsOut, pCaCookiesOut, (LPTSTR) szFlashType, 0);
		case DISPID_BC_MOUSEPOINTER: //MousePointer
			return GetEnumList(pCaStringsOut, pCaCookiesOut, (LPTSTR) szMousePointer, 0);

		case DISPID_BC_ENABLED: //Enabled
		case DISPID_BC_WORDWRAP: //WordWrap
			return GetEnumList(pCaStringsOut, pCaCookiesOut, (LPTSTR) sztruefalse, -1);
		default:
			return E_NOTIMPL;
		}
	}
	return E_NOTIMPL;
}

STDMETHODIMP Cbclabel::GetPredefinedValue(DISPID dispID, DWORD dwCookie, VARIANT* pVarOut)
{
	switch (dispID)
	{
		case DISPID_BC_ALIGNMENT: //Alignment
		case DISPID_BC_VALIGNMENT: //VAlignment
		case DISPID_BC_BACKSTYLE: //BackStyle
		case DISPID_BC_BORDERSTYLE: //BorderStyle
		case DISPID_BC_FLASHTYPE: //FlashType
		case DISPID_BC_MOUSEPOINTER: //MousePointer

		case DISPID_BC_ENABLED: //Enabled
		case DISPID_BC_WORDWRAP: //WordWrap
			VariantClear(pVarOut);
			V_VT(pVarOut) = VT_I2;
			V_I2(pVarOut) = (short)dwCookie;
			return S_OK;
		default:
			return E_NOTIMPL;
	}
}


STDMETHODIMP Cbclabel::MapPropertyToCategory(/*[in]*/ DISPID dispid, /*[out]*/ PROPCAT* ppropcat)
{
    if (NULL == ppropcat) return E_POINTER;

    switch (dispid) 
	{
		case DISPID_BC_ALIGNMENT:
		case DISPID_BC_VALIGNMENT:
		case DISPID_BC_BACKCOLOR:
		case DISPID_BC_BACKSTYLE:
		case DISPID_BC_BORDERCOLOR:
		case DISPID_BC_BORDERSTYLE:
		case DISPID_BC_BORDERSIZE:
		case DISPID_BC_FORECOLOR:
			*ppropcat = PROPCAT_Appearance;
			return S_OK;

		case DISPID_BC_WORDWRAP:
		case DISPID_BC_CAPTION:
			*ppropcat = PROPCAT_Text;
			return S_OK;

		case DISPID_BC_HELPCONTEXTID:
		case DISPID_BC_MOUSEICON:
		case DISPID_BC_MOUSEPOINTER:
		case DISPID_BC_ENABLED:
			*ppropcat = PROPCAT_Behavior;
			return S_OK;

		case DISPID_BC_FONT:
			*ppropcat = PROPCAT_Font;
			return S_OK;

		case DISPID_BC_FLASHTIME:
		case DISPID_BC_FLASHCOLOR:
		case DISPID_BC_FLASHTYPE:
			*ppropcat = PROPCAT_Flashing;
			return S_OK;

		default:
			return E_FAIL;
    }
}

STDMETHODIMP Cbclabel::GetCategoryName(/*[in]*/ PROPCAT propcat, /*[in]*/ LCID lcid, /*[out]*/ BSTR* pbstrName)
{
    if(PROPCAT_Flashing == propcat) 
	{
        *pbstrName = ::SysAllocString(L"Flashing"); 
        return S_OK;
    }
    return E_FAIL;
}


HRESULT Cbclabel::BCFireOnRequestEdit(DISPID id)
{
	//if (!m_nFreezeEvents) 
		if (FireOnRequestEdit(id) == S_FALSE)
			return S_FALSE;
	return S_OK;
}
void Cbclabel::SomethingChanged(DISPID id)
{
	m_bRequiresSave = TRUE;
	//if (!m_nFreezeEvents) 
		FireOnChanged(id);
	FireViewChange();
	SendOnDataChange(NULL);
}

STDMETHODIMP Cbclabel::About()
{
	//TODO: add functionality
	return S_OK;
}

STDMETHODIMP Cbclabel::Refresh()
{
	FireViewChange();
	return S_OK;
}

STDMETHODIMP Cbclabel::Zorder(VARIANT newValue)
{
	if (newValue.intVal == 0)
		::SetWindowPos(m_hWnd,HWND_TOP,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE|SWP_FRAMECHANGED);
	else
		::SetWindowPos(m_hWnd,HWND_BOTTOM,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE|SWP_FRAMECHANGED);

	return S_OK;
}

STDMETHODIMP Cbclabel::get_Alignment(/*[out, retval]*/ enumAlignment* pVal)
{
	*pVal = (enumAlignment)m_alignment;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_Alignment(/*[in]*/ enumAlignment newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_ALIGNMENT) == S_FALSE)
		return S_FALSE;
	m_alignment = newVal;
	SomethingChanged(DISPID_BC_ALIGNMENT);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_VAlignment(/*[out, retval]*/ enumVAlignment* pVal)
{
	*pVal = (enumVAlignment) m_valignment;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_VAlignment(/*[in]*/ enumVAlignment newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_VALIGNMENT) == S_FALSE)
		return S_FALSE;
	m_valignment = newVal;
	SomethingChanged(DISPID_BC_VALIGNMENT);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_BackColor(/*[out, retval]*/ OLE_COLOR* pVal)
{
	*pVal = m_backcolor;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_BackColor(/*[in]*/ OLE_COLOR newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_BACKCOLOR) == S_FALSE)
		return S_FALSE;
	m_backcolor = newVal;
	SomethingChanged(DISPID_BC_BACKCOLOR);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_BackStyle(/*[out, retval]*/ enumBackStyle* pVal)
{
	*pVal = (enumBackStyle) m_backstyle;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_BackStyle(/*[in]*/ enumBackStyle newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_BORDERSTYLE) == S_FALSE)
		return S_FALSE;
	m_backstyle = newVal;
	SomethingChanged(DISPID_BC_BORDERSTYLE);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_BorderColor(/*[out, retval]*/ OLE_COLOR* pVal)
{
	*pVal = m_bordercolor;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_BorderColor(/*[in]*/ OLE_COLOR newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_BORDERCOLOR) == S_FALSE)
		return S_FALSE;
	m_bordercolor = newVal;
	SomethingChanged(DISPID_BC_BORDERCOLOR);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_BorderStyle(/*[out, retval]*/ enumBorderStyle* pVal)
{
	*pVal = (enumBorderStyle) m_borderstyle;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_BorderStyle(/*[in]*/ enumBorderStyle newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_BORDERSTYLE) == S_FALSE)
		return S_FALSE;
	m_borderstyle = newVal;
	SomethingChanged(DISPID_BC_BORDERSTYLE);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_BorderSize(/*[out, retval]*/ float* pVal)
{
	USES_CONVERSION;
	SIZEL	inpix;
	POINTF	incont;
	CComBSTR	cbstrscale=_T("Twip");

	inpix.cx=m_bordersize;
	inpix.cy=m_bordersize;
	GetAmbientScaleUnits(cbstrscale.m_str);
	BCPixeltoContainerSize(m_polecontrolsite, &inpix, &incont, OLE2T(cbstrscale.m_str));
	*pVal = incont.x;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_BorderSize(/*[in]*/ float newVal)
{
	SIZEL	inpix;
	POINTF	incont;
	if (BCFireOnRequestEdit(DISPID_BC_BORDERSIZE) == S_FALSE)
		return S_FALSE;
	incont.x=newVal;
	incont.y=newVal;
	BCContainertoPixelSize(m_polecontrolsite, &inpix, &incont);
	m_bordersize = inpix.cx;
	SomethingChanged(DISPID_BC_BORDERSIZE);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_Caption(/*[out, retval]*/ BSTR* pVal)
{
	*pVal = m_caption.Copy();
	return S_OK;
}

STDMETHODIMP Cbclabel::put_Caption(/*[in]*/ BSTR newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_CAPTION) == S_FALSE)
		return S_FALSE;
	m_caption = newVal;
	SomethingChanged(DISPID_BC_CAPTION);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_Enabled(/*[out, retval]*/ VARIANT_BOOL* pVal)
{
	*pVal = m_enabled;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_Enabled(/*[in]*/ VARIANT_BOOL newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_ENABLED) == S_FALSE)
		return S_FALSE;
	m_enabled = newVal;
	SomethingChanged(DISPID_BC_ENABLED);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_FlashColor(/*[out, retval]*/ OLE_COLOR* pVal)
{
	*pVal = m_flashcolor;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_FlashColor(/*[in]*/ OLE_COLOR newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_FLASHCOLOR) == S_FALSE)
		return S_FALSE;
	m_flashcolor = newVal;
	SomethingChanged(DISPID_BC_FLASHCOLOR);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_FlashType(/*[out, retval]*/ enumFlashType* pVal)
{
	*pVal = (enumFlashType) m_flashtype;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_FlashType(/*[in]*/ enumFlashType newVal)
{
	BOOL bMode=FALSE;
	GetAmbientUserMode(bMode);
	
	if (BCFireOnRequestEdit(DISPID_BC_FLASHTYPE) == S_FALSE)
		return S_FALSE;
	m_flashtype = newVal;
	if (bMode)           // if run mode...
	{
		if (m_flashtype > 0)
		{
			//set the timer
			if (::IsWindow(m_hWnd))
			{
				::SetTimer(m_hWnd,1,200,NULL);                    
				//set the time for comparison
				m_lastflash = GetTickCount();
			}
		}
	}
	SomethingChanged(DISPID_BC_FLASHTYPE);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_Font(/*[out, retval]*/ IFontDisp** ppFont)
{
	*ppFont = m_pfont;
	if (*ppFont != NULL)
		(*ppFont)->AddRef();
	return S_OK;
}

STDMETHODIMP Cbclabel::putref_Font(/*[in, out]*/ IFontDisp* pFont)
{
	if (BCFireOnRequestEdit(DISPID_BC_FONT) == S_FALSE)
		return S_FALSE;
	m_pfont = pFont;
	CComQIPtr<IFont,&IID_IFont> pFont1(m_pfont);
	if (pFont1)
		pFont1->get_hFont(&m_hfont);
	SomethingChanged(DISPID_BC_FONT);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_ForeColor(/*[out, retval]*/ OLE_COLOR* pVal)
{
	*pVal = m_forecolor;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_ForeColor(/*[in]*/ OLE_COLOR newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_FORECOLOR) == S_FALSE)
		return S_FALSE;
	m_forecolor = newVal;
	SomethingChanged(DISPID_BC_FORECOLOR);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_HelpContextID(/*[out, retval]*/ long* pVal)
{
	*pVal = m_helpcontextid;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_HelpContextID(/*[in]*/ long newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_HELPCONTEXTID) == S_FALSE)
		return S_FALSE;
	m_helpcontextid = newVal;
	SomethingChanged(DISPID_BC_HELPCONTEXTID);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_MouseIcon(/*[out, retval]*/ IPictureDisp** pVal)
{
	*pVal = m_pmouseicon;
	if (*pVal != NULL)
		(*pVal)->AddRef();
	return S_OK;
}

STDMETHODIMP Cbclabel::putref_MouseIcon(/*[in, out]*/ IPictureDisp* newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_MOUSEICON) == S_FALSE)
		return S_FALSE;
	m_pmouseicon = newVal;
	SomethingChanged(DISPID_BC_MOUSEICON);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_MousePointer(/*[out, retval]*/ enumMousePointer* pVal)
{
	*pVal = (enumMousePointer)m_mousepointer;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_MousePointer(/*[in]*/ enumMousePointer newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_MOUSEPOINTER) == S_FALSE)
		return S_FALSE;
	m_mousepointer = newVal;
	SomethingChanged(DISPID_BC_MOUSEPOINTER);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_WordWrap(/*[out, retval]*/ VARIANT_BOOL* pVal)
{
	*pVal = m_wordwrap;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_WordWrap(/*[in]*/ VARIANT_BOOL newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_WORDWRAP) == S_FALSE)
		return S_FALSE;
	m_wordwrap = newVal;
	SomethingChanged(DISPID_BC_WORDWRAP);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_FlashTime(/*[out, retval]*/ long* pVal)
{
	*pVal = m_flashtime;
	return S_OK;
}

STDMETHODIMP Cbclabel::put_FlashTime(/*[in]*/ long newVal)
{
	if (newVal < 0)
		return Error(_T("Invalid property value"),CLSID_bclabel,0x800A017C);		
	if (BCFireOnRequestEdit(DISPID_BC_FLASHTIME) == S_FALSE)
		return S_FALSE;
	m_flashtime = newVal;
	SomethingChanged(DISPID_BC_FLASHTIME);
	return S_OK;
}

STDMETHODIMP Cbclabel::put_BorderSizePix(/*[in]*/ long newVal)
{
	if (BCFireOnRequestEdit(DISPID_BC_BORDERSIZEPIX) == S_FALSE)
		return S_FALSE;
	m_bordersize = newVal;
	SomethingChanged(DISPID_BC_BORDERSIZEPIX);
	return S_OK;
}

STDMETHODIMP Cbclabel::get_BorderSizePix(/*[out, retval]*/ long *pVal)
{
	*pVal = m_bordersize;
	return S_OK;
}

STDMETHODIMP Cbclabel::SetClientSite (LPOLECLIENTSITE pSite)
{      
	HRESULT hr = CComControlBase::IOleObject_SetClientSite(pSite);
	// Check to see if the container has an ambient font. If it does,
	// clone it so your user can change the font of the control
	// without changing the ambient font for the container. If there is
	// no ambient font, create your own font object when you hook up a
	// client site.      if(!m_pFont && pSite)      {
	FONTDESC fd = _fontdesc;
	CComPtr<IFont> pAF;
	CComPtr<IFont> pClone;
	LPVOID pInterface;
	LPVOID pInterface1;
	CComBSTR	cbstrscale=_T("Twip");

	if(SUCCEEDED(GetAmbientFont(&pAF)))       
	{
           //clone the font          
		if(SUCCEEDED(pAF->Clone(&pClone)))
              pClone->QueryInterface(IID_IFontDisp, (void**)&m_pfont);
	}
       else       
	{
          OleCreateFontIndirect(&fd,IID_IFontDisp,(void**)&(m_pfont.p));
	}
	CComQIPtr<IFont,&IID_IFont> pFont1(m_pfont);
	if (pFont1)
	{
		pFont1->get_hFont(&m_hfont);
	}
	
	// Initialize pointer to simple frame site
	if (pSite == NULL || 
		FAILED(pSite->QueryInterface(IID_ISimpleFrameSite, &pInterface)))
	{
		pInterface = NULL;
	}

	//init pointer to ole control site
	if (pSite == NULL || 
		FAILED(pSite->QueryInterface(IID_IOleControlSite, &pInterface1)))
	{
		pInterface1 = NULL;
	}

	if (m_polecontrolsite)
		m_polecontrolsite->Release();
	m_polecontrolsite = (LPOLECONTROLSITE) pInterface1;
	return hr;
}

LRESULT Cbclabel::OnKey(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	bHandled = FALSE;
	return 0;
}

LRESULT Cbclabel::OnMMessage(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	
	USES_CONVERSION;
	short	button;
	short	shift = GetShiftState();
	float	X;
	float	Y;
	HDC		hdc=::GetDC(m_hWnd);
	POINT	pt;

	
	
	

	//X = (float) XPixelsToTwips((int) (short) LOWORD(lParam), NULL);
	//Y = (float) YPixelsToTwips((int) (short) HIWORD(lParam), NULL);

	//cbstrscale = SysAllocString((const OLECHAR *)_T("Twip"));
	//GetAmbientScaleUnits((BSTR&)cbstrscale);
	//tsize.cx=(int) (short) LOWORD(lParam);
	//tsize.cy=(int) (short) HIWORD(lParam);
	//psize.x=0;
	//psize.y=0;
	//BCPixeltoContainerPos(m_polecontrolsite, &tsize, &psize, OLE2T(cbstrscale));
	//SysFreeString(cbstrscale);
	//X=psize.x;
	//Y=psize.y;

	pt.x=(int) (short) LOWORD(lParam);
	pt.y=(int) (short) HIWORD(lParam);
	DPtoLP(hdc,&pt,1);

	X=(float)pt.x;
	Y=(float)pt.y;
	::ReleaseDC(m_hWnd,hdc);

	switch (uMsg)
	{
	case WM_LBUTTONDBLCLK:
		Fire_DblClick();
		break;
	case WM_LBUTTONDOWN:
		button = 1;
		m_sButton = 1;
		Fire_MouseDown((LONG) &button, (LONG) &shift, (LONG) &X, (LONG) &Y);
		SetCapture();
		break;
	case WM_LBUTTONUP:
		button = 1;
		m_sButton = 1;
		ReleaseCapture();
		Fire_MouseUp((LONG) &button, (LONG) &shift, (LONG) &X, (LONG) &Y);
		Fire_Click();
		break;
	case WM_MOUSEMOVE:
		if (!m_overcontrol)
			::PostMessage(m_hWnd,WM_TIMER,0,0L);
		button = m_sButton;
		Fire_MouseMove((LONG) &button, (LONG) &shift, (LONG) &X, (LONG) &Y);
		break;
	case WM_RBUTTONDOWN:
		button = 2;
		m_sButton = 2;
		Fire_MouseDown((LONG) &button, (LONG) &shift, (LONG) &X, (LONG) &Y);
		SetCapture();
		break;
	case WM_RBUTTONUP:
		button = 2;
		m_sButton = 2;
		ReleaseCapture();
		Fire_MouseUp((LONG) &button, (LONG) &shift, (LONG) &X, (LONG) &Y);
		break;
	case WM_NCHITTEST:
		{
		BOOL bMode=FALSE;

		GetAmbientUserMode(bMode);

		if (bMode)
			if (!m_overcontrol)
				::PostMessage(m_hWnd,WM_TIMER,0,0L);
		break;
		}
	}

  // TODO : Add Code for message handler. Call DefWindowProc if necessary.
	bHandled = FALSE;
	return 0;
}
LRESULT Cbclabel::OnSetCursor(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	POINT ptMouse;
	HCURSOR hCursor = NULL;
	HINSTANCE hInstance = NULL;
	BOOL bMode=FALSE;

	//TODO: animated stuff
	GetAmbientUserMode(bMode);

	// If the MousePointer property does not indicate the default 
	// cursor

	if (m_mousepointer && bMode)
	{
		// If the mouse cursor is over the client area

		if (((HWND)wParam == m_hWnd) && (LOWORD(lParam) == HTCLIENT))
		{
			GetCursorPos(&ptMouse);
			::ScreenToClient(m_hWnd, &ptMouse);

			// If the MousePointer property indicates a custom 	
			//cursor is not to be used, set the mouse cursor to a 
			//stock bitmap. 
			// Otherwise, use the bitmap indicated by the MouseIcon 
			// property.
	
			if ((m_mousepointer != CUSTOM_CURSOR) && (m_mousepointer != 16))
			{
				hCursor = LoadCursor(hInstance, idCursor[m_mousepointer]);
			}
			else if (m_pmouseicon)
			{
				CComQIPtr<IPicture,&IID_IPicture> pPic(m_pmouseicon);
				if (pPic)
				{
					pPic->get_Handle((OLE_HANDLE *)&hCursor);
				}
			}

			// If the bitmap exists, use it to set the mouse cursor
		
			if (hCursor)
				SetCursor(hCursor);
		
		return 0;
		}
	}
	bHandled=FALSE;
	return 0;
}

LRESULT Cbclabel::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	BOOL bMode=FALSE;
	GetAmbientUserMode(bMode);
	
	if (bMode)           // if run mode...
	{
		if (m_flashtype > 0)
		{
			//set the timer
			if (::IsWindow(m_hWnd))
			{
				::SetTimer(m_hWnd,1,200,NULL);                    
				//set the time for comparison
				m_lastflash = GetTickCount();
			}
		}
	}
	return S_OK;
}

LRESULT Cbclabel::OnEraseBK(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	//just skip to avoid flash
	bHandled=TRUE;
	return S_OK;
}

LRESULT Cbclabel::OnTimer(UINT uMsg, WPARAM wp, LPARAM lp, BOOL& bHandled)
{
	POINT 	Point;
	BOOL inrect;
	RECT	wrect;
	BOOL bMode=FALSE;
	GetAmbientUserMode(bMode);
	
                    
	//no tip unless in run mode
	if (!bMode)           // if not run mode...
		return 0;
	
	//if window is disabled then return!
	if (!m_enabled)
		return 0;

	if (m_flashtype > 0)
	{
		//when is the last time we flashed it
		//is it time to do it again
		//handle flashing the text
		if (GetTickCount() > (m_lastflash + m_flashtime))
		{
			if (m_curflash)
				m_curflash=0;
			else
				m_curflash=-1;
			FireViewChange();
			m_lastflash=GetTickCount();
		}
	}
	GetCursorPos( &Point);
	//get our screen rect
	GetWindowRect(&wrect);
	//if we're in it set to true and fire event
	if ((Point.x >= wrect.left) &&
		(Point.x <= wrect.right) &&
		(Point.y >= wrect.top) &&
		(Point.y <= wrect.bottom))
		inrect=TRUE;

	if (TRUE == inrect)
	{
		if (!m_overcontrol)
		{
			m_overcontrol = TRUE;
			Fire_MouseEnter();
			FireViewChange();
			::SetWindowPos(m_hWnd,NULL,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE|SWP_NOZORDER|SWP_FRAMECHANGED);
			if (m_flashtype == 0)
				::SetTimer(m_hWnd,1,200,NULL);
		}
	}
	else
	{
		if (m_overcontrol)
		{
			m_overcontrol = FALSE;
			Fire_MouseExit();
			FireViewChange();
			::SetWindowPos(m_hWnd,NULL,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE|SWP_NOZORDER|SWP_FRAMECHANGED);
			if (m_flashtype == 0)
				::KillTimer(m_hWnd,1);
		}
	}
	return 0;

}


HRESULT Cbclabel::OnDraw(ATL_DRAWINFO& di)
{
	USES_CONVERSION;
	RECT			rect;
	HDC				hdc;
    HFONT			oldfont;
    LPTSTR          lpmem;
    WORD            wFormat;
	long			bordersize;
	enumVAlignment	valignment;
	CComBSTR		caption;
	OLE_COLOR		bordercolor;
	enumAlignment	alignment;
	enumBorderStyle	borderstyle;
	short			wordwrap;
	OLE_COLOR		forecolor = m_forecolor;

	bordersize = m_bordersize;
	borderstyle = (enumBorderStyle)m_borderstyle;
	bordercolor = m_bordercolor;
	caption = m_caption.Copy();
	valignment = (enumVAlignment)m_valignment;
	alignment = (enumAlignment)m_alignment;
	wordwrap = m_wordwrap;
	
	if (m_curflash)
	{
		switch (m_flashtype)
		{
		case 1:
			forecolor = 0;
			break;
		case 2:
			forecolor = m_backcolor;
			break;
		case 3:
			forecolor = m_flashcolor;
			break;
		default:
			;
		}
	}
	hdc = ::GetWindowDC(m_hWnd);
	::GetWindowRect(m_hWnd, &rect);
	//normalize the rect
	rect.right-=rect.left;
	rect.bottom-=rect.top;
	rect.left=0;
	rect.top=0;

	BCDrawBorder(
		m_hWnd,    		//the hwnd to draw in
		hdc,			//the hdc to draw on
		(short)m_bordersize,	//the width to draw
		borderstyle,	//do we draw a border around the border?
		-1,				//fill in the center?
		m_backcolor,	//fill color
		bordercolor,	//border color
		ODA_DRAWENTIRE,
		&rect);

    if (m_hfont)
    {
		DWORD	oldcolor=0;
		BOOL    bEnabled=FALSE;
		
        oldcolor = SetBkColor(hdc, GizmosGetVBSysColors(m_backcolor));
        ::GetWindowRect(m_hWnd, &rect);
		rect.right-=rect.left;
		rect.bottom-=rect.top;
		rect.left=0;
		rect.top=0;
		rect.left = rect.left + bordersize + 2;
		rect.right = rect.right - bordersize - 2;
		rect.top = rect.top + bordersize + 2;
		rect.bottom = rect.bottom - bordersize - 2;
        oldfont = (HFONT) SelectObject(hdc,m_hfont);
        SetBkMode(hdc, TRANSPARENT);
        if (caption.Length()>0)
        {
            lpmem= OLE2T(caption.m_str);
            wFormat = 0;
			if (wordwrap)
				wFormat = wFormat | DT_WORDBREAK;
			else
				wFormat = wFormat | DT_SINGLELINE;
            switch (alignment)
                {
                case 0:
                    wFormat = wFormat | DT_LEFT;
                    break;
                case 1:
                    wFormat = wFormat | DT_RIGHT;
                    break;
                case 2:
                    wFormat = wFormat | DT_CENTER;        
                }                                 
                
            switch (valignment)
                {
                case 0:
                    wFormat = wFormat | DT_TOP;
                    break;
                case 1:
                    wFormat = wFormat | DT_BOTTOM;
                    break;
                case 2:
                    wFormat = wFormat | DT_VCENTER;        
                }                                 
            bEnabled = m_enabled;
			               
            if ((!::IsWindowEnabled(m_hWnd)||!bEnabled))
			{
				LONG	disabledcolor = GetSysColor(COLOR_GRAYTEXT);

				if (GetSysColor(COLOR_GRAYTEXT) == (DWORD) GizmosGetVBSysColors(m_backcolor))
					disabledcolor = 0x00808080;
				SetTextColor(hdc, GizmosGetVBSysColors(disabledcolor));
				DrawText(hdc, lpmem, -1, &rect, wFormat);
			}
			else
			{
				SetTextColor(hdc, GizmosGetVBSysColors(forecolor));
				DrawText(hdc, lpmem, -1, &rect, wFormat);
			}
        }
       	SetBkColor(hdc, oldcolor);
        SelectObject(hdc,oldfont);
    }
	ReleaseDC(hdc);
	return S_OK;
}
