namespace netLabel
{
	using System;
	using System.Collections;
	using System.ComponentModel;
	using System.Drawing;
	using System.Data;
	using System.Windows.Forms;
	using System.ComponentModel.Design;
	using System.Diagnostics;
	using System.Drawing.Drawing2D;
	using System.Drawing.Design;
	using System.Windows.Forms.Design;
	
	/// <summary>
	///		This declares a delegate which is used in an event.
	///		The timer names are a little confusing, because some
	///		are for the timer to make the border flash
	///		others are for the event that we generate
	/// </summary>
	public delegate void TickTimerHandler(object sender, TickTimerEventArgs e);
	
	
	/// <summary>
	///		an exception class to throw an error
	///		I still have to refine this
	/// </summary>
	public class InvalidPropertyValueException : Exception
	{
		private readonly string message="Invalid property Value";

		public InvalidPropertyValueException() {}
		public InvalidPropertyValueException(string message) {  }
		
		new public string Message 
		{
			get 
			{
				return message;
			}
		}
	}

	/// <summary>
	///		everything is a class
	///		this is the arguments for a custom event
	///		I think VB still works with the Event keyword
	///		they should have used that for C#, this sucks
	/// </summary>
	public class TickTimerEventArgs : EventArgs 
	{
		//a cancel paramter to stop the flashing
		private bool bCancel = false;
		//the border color
		private Color cBorderColor = System.Drawing.SystemColors.WindowFrame;
		//event args constructor
		public TickTimerEventArgs(bool bCancel, Color cBorderColor) {}
		
		//these are properties that set the member variables
		public bool Cancel
		{ 
			get 
			{ 
				return bCancel;
			}
			set
			{
				bCancel = value;
			}
		}
		public Color BorderColor
		{ 
			get 
			{ 
				return cBorderColor;
			}
			set
			{
				cBorderColor=value;
			}
		}    
	}
	//property types
	public enum enumAlignment
	{
		Left,
		Center,
		Right
	};

	public enum enumBorderStyle
	{
		No_Border=0,
		ThreeD_Border=1,
		Single_Color_Border=2
	};

	public enum enumBackStyle
	{
		Transparent = 0,
		Opaque = 1
	};

	public enum enumFlashType
	{
		NoneFlash = 0,
		BlackFlash = 1,
		BackgroundFlash = 2,
		ColorFlash = 3
	};

	public enum enumVAlignment
	{
		Top,
		Center,
		Bottom
	};

	//ah, finally the Control
	/// <summary>
    ///    Summary description for nLabel.
    /// </summary>
	public class nLabel : System.Windows.Forms.UserControl
    {
		private System.ComponentModel.IContainer components;
		
		/// <summary>
		///		several public enmus, used for properties
		///		these items appear in the selection list
		///		in the property browser for the properties
		///		no apparent way to make nice readable text 
		///		in the lists.  Perhaps VB allows the []
		///		syntax that allows spaces in the var 
		///		names.  C# doesn't
		/// </summary>
		private enumAlignment		m_alignment;
		private enumVAlignment		m_valignment;
		private enumBackStyle		m_backstyle;
		private Color				m_bordercolor;
		private enumBorderStyle		m_borderstyle;
		private int					m_bordersize;
		private Color		 		m_flashcolor;
		private enumFlashType		m_flashtype;
		private bool				m_wordwrap;
		private int					m_flashtime;
		private Color				m_curflash;
		//supplies the timer that allows the
		//border to flash
		Timer						m_timer;
        //the timer event that fires from this control
		private TickTimerHandler	m_eTimer; 

		//properties
		//Alignment
		[
			Category("Appearance"),
			Description("Specifies the horizontal alignment of text."),
			Bindable(true),
			Browsable(true)
		]
		public enumAlignment Alignment 
		{
			get 
			{
				return m_alignment;
			}
			set 
			{
				m_alignment = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
		//VAlignment
		[
			Category("Appearance"),
			Description("Specifies the vertical alignment of text."),
			Browsable(true),
			DefaultValue(enumVAlignment.Top)
		]
		public enumVAlignment VAlignment 
		{
			get 
			{
				return m_valignment;
			}
			set 
			{
				m_valignment = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
		//Caption
		[
			Category("Appearance"),
			Description("Specifies the caption."),
		]
		public string Caption 
		{
			get 
			{
				return Text;
			}
			set 
			{
				Text = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        

        [
        Browsable(false)
        ]
        public override string Text 
		{
            get 
			{
                return base.Text;
            }
            set 
			{
                base.Text = value;
            }
			
        }

		//BackStyle
		[
			Category("Appearance"),
			Description("Specifies the tranparancy of the control."),
			Browsable(true),
			DefaultValue(enumBackStyle.Opaque)
		]
		public enumBackStyle BackStyle 
		{
			get 
			{
				return m_backstyle;
			}
			set 
			{
				m_backstyle = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //BorderColor
		[
			Category("Appearance"),
			Description("Specifies the color of the border."),
			Browsable(true)
		]
		public Color BorderColor 
		{
			get 
			{
				return m_bordercolor;
			}
			set 
			{
				m_bordercolor = value;
				m_curflash = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //BorderStyle
		[
			Category("Appearance"),
			Description("Specifies the style of the border."),
			Browsable(true),
			DefaultValue(enumBorderStyle.Single_Color_Border)
		]
		public enumBorderStyle BorderStyle
		{
			get 
			{
				return m_borderstyle;
			}
			set 
			{
				m_borderstyle = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
		
		//BorderSize
		[
			Category("Appearance"),
			Description("Specifies the size of the border."),
			Browsable(true),
			DefaultValue(1)
		]
		public int BorderSize
		{
			get 
			{
				return m_bordersize;
			}
			set 
			{
				if (value>10)
				{
					InvalidPropertyValueException ex=new InvalidPropertyValueException();
					throw ex;
				}
				m_bordersize = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //FlashColor
		[
			Category("Appearance"),
			Description("Specifies the color that flashes."),
			Browsable(true)
		]
		public Color FlashColor 
		{
			get 
			{
				return m_flashcolor;
			}
			set 
			{
				m_flashcolor = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //FlashType
		[
			Category("Appearance"),
			Description("Specifies the type of flashing."),
			Browsable(true),
			DesignerSerializationVisibility(DesignerSerializationVisibility.Visible),
			DefaultValue(enumFlashType.NoneFlash)
		]
		public enumFlashType FlashType
		{
			get 
			{
				return m_flashtype;
			}
			set 
			{
				m_flashtype = value;
				//Invalidate invokes the OnPaint method
				//Invalidate();
				if (m_flashtype>0)
				{
					if (m_flashtime>0)
					{
						if (DesignMode==false)
						{
							m_timer.Start();
						}
					}
				}
			}
		}
        //MouseIcon
        //MousePointer

		//WordWrap
		[
			Category("Appearance"),
			Description("Determines if the text wraps or not."),
			Browsable(true),
			//Persistable(PersistableSupport.All),
			DefaultValue(true)
		]
		public bool WordWrap
		{
			get 
			{
				return m_wordwrap;
			}
			set 
			{
				m_wordwrap = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //FlashTime
		[
			Category("Appearance"),
			Description("Specifies the time between flashes."),
			Browsable(true),
			DefaultValue(0)
		]
		public int FlashTime
		{
			get 
			{
				return m_flashtime;
			}
			set 
			{
				if (m_flashtype!=enumFlashType.NoneFlash)
				{
					m_timer.Stop();
				}
				m_flashtime = value;
				if (value>0)
				{
					m_timer.Interval=value;
					if (m_flashtype>0)
					{
						if (DesignMode==false)
						{
							m_timer.Start();
						}
						
					}
				}
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}

        //Timer Event
        [Description("Raised when the Timer message hits")]
        //this is a property that holds a reference to the
		//event.  Think of it as a type safe pointer
		public event TickTimerHandler TickTimer {
            add 
			{
                m_eTimer += value;
            }
            remove 
			{
                m_eTimer -= value;
            }
        }
        //this is where the event is actually fired
		//we could fire the event directly, without this
		//function, but using this virtual function
		//allows controls based on this class to override 
		//this event
		protected virtual void OnTimer(TickTimerEventArgs e) 
		{
            if (m_eTimer != null) 
				m_eTimer(this, e);
        }



		public override void  Refresh()
		{
			Invalidate();
		}
		public nLabel()
        {
            InitializeComponent();
            // Add any initialization after the InitForm call

        }

        protected override void OnPaint(System.Windows.Forms.PaintEventArgs sa)
		{
			StringFormat style = new StringFormat();
			//provides left to right alignment
			switch (m_alignment)
			{
				case enumAlignment.Left:
					style.Alignment=System.Drawing.StringAlignment.Near;
					break;
				case enumAlignment.Center:
					style.Alignment=System.Drawing.StringAlignment.Center;
					break;
				case enumAlignment.Right:
					style.Alignment=System.Drawing.StringAlignment.Far;
					break;
				default:
					style.Alignment=System.Drawing.StringAlignment.Near;
					break;
			}
			//provide top to bottom alignment
			switch (m_valignment)
			{
				case enumVAlignment.Top:
					style.LineAlignment=System.Drawing.StringAlignment.Near;
					break;
				case enumVAlignment.Center:
					style.LineAlignment=System.Drawing.StringAlignment.Center;
					break;
				case enumVAlignment.Bottom:
					style.LineAlignment=System.Drawing.StringAlignment.Far;
					break;
				default:
					style.LineAlignment=System.Drawing.StringAlignment.Near;
					break;
			}

			sa.Graphics.DrawRectangle(new Pen(m_curflash,1),ClientRectangle.Top,ClientRectangle.Left,ClientRectangle.Width,ClientRectangle.Height);
			sa.Graphics.DrawString(Text,Font,new SolidBrush(ForeColor),ClientRectangle,style);

		}
		/// <summary>
        ///    Clean up any resources being used.
        /// </summary>
/*        public override void Dispose()
        {
            base.Dispose();
            components.Dispose();
        }
*/
        /// <summary>
        ///    Required method for Designer support - do not modify 
        ///    the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.m_timer = new System.Windows.Forms.Timer(this.components);
			// 
			// m_timer
			// 
			this.m_timer.Tick += new System.EventHandler(this.OnTimerEvent);
			// 
			// nLabel
			// 
			this.AccessibleName = "netLabel";
			this.Name = "nLabel";
			this.Size = new System.Drawing.Size(344, 80);
			this.Load += new System.EventHandler(this.nLabel_Load);

		}

		/// <summary>
		///		this is where we recieve our timer event
		///		we use this to flash the border
		/// </summary>
		private void OnTimerEvent(object source, EventArgs e)
		{
			//we create the arguments for our own event
			TickTimerEventArgs	etick = new TickTimerEventArgs(false,m_curflash);
			//and fire our event
			OnTimer(etick);
			//if we weren't canceled in our event
			//then do the flash
			if (!etick.Cancel)
			{
					
				if (m_curflash==m_bordercolor)
				{
					m_curflash=m_flashcolor;
				}
				else
				{
					m_curflash=m_bordercolor;
				}
				Invalidate();
			}
		}

		private void nLabel_Load(object sender, System.EventArgs e)
		{

		}
    }
}
