namespace netLabel
{
    using System;
    using System.Collections;
    using System.Core;
    using System.ComponentModel;
    using System.Drawing;
    using System.Data;
    using System.WinForms;
    using System.ComponentModel.Design;
    using System.Diagnostics;
    using System.Drawing. Drawing2D;
    using System.Drawing.Design;
    using System.WinForms.ComponentModel;
    using System.WinForms.Design;
	
	public delegate void TickTimerHandler(object sender, TickTimerEventArgs e);
	
    public class BorderSizeEditor : UITypeEditor 
	{
    
        private IWinFormsEditorService edSvc = null;
    
        public override object EditValue(ITypeDescriptorContext context, IServiceObjectProvider provider, object value) 
		{
    
            if (context != null && context.Instance != null && provider != null) 
			{
    
                edSvc = (IWinFormsEditorService)provider.GetServiceObject(typeof(IWinFormsEditorService));
    
                if (edSvc != null) 
				{
                    TrackBar trackBar = new TrackBar();
                    trackBar.ValueChanged += new EventHandler(this.ValueChanged);
					trackBar.ForeColor = Color.White;
					trackBar.Minimum = 0;
					trackBar.Maximum = 20;
					trackBar.Height = 23;
                    bool asInt = true;
                    if (value is int) 
					{
                        trackBar.Value = (int)value;
                    }
                    else if (value is byte) 
					{
                        asInt = false;
                        trackBar.Value = (byte)value;
                    }
                    edSvc.DropDownControl(trackBar);
                    if (asInt) 
					{
                        value = trackBar.Value;
                    }
                    else 
					{
                        value = (byte)trackBar.Value;
                    }
                }
            }
    
            return value;
        }

    
        public override UITypeEditorEditStyle  
        GetEditStyle(ITypeDescriptorContext context) 
		{
            if (context != null && context.Instance != null) 
			{
                return UITypeEditorEditStyle.DropDown;
            }
            return base.GetEditStyle(context);
        }
    
        private void ValueChanged(object sender, EventArgs e) 
		{
			if (edSvc != null) 
			{
                edSvc.CloseDropDown();
            }
        }
    }


	
	public class InvalidPropertyValueException : Exception
	{
		private readonly string message="Invalid property Value";

	    public InvalidPropertyValueException() {}
		public InvalidPropertyValueException(string message) {  }
		
        public string Message 
		{
            override get 
			{
                return message;
            }
        }
	}

	public class TickTimerEventArgs : EventArgs 
	{
   
		private bool bCancel = false;
		private Color cBorderColor = System.Drawing.SystemColors.WindowFrame;
		//constructor
		public TickTimerEventArgs(bool bCancel, Color cBorderColor) {}
		//properties
		public bool Cancel
		{ 
			get 
			{ 
				return bCancel;
			}
			set
			{
				bCancel = value;
			}
		}
		public Color BorderColor
		{ 
			get 
			{ 
				return cBorderColor;
			}
			set
			{
				cBorderColor=value;
			}
		}    
   }
   
	
	/*public class GraphicsBC : System.Drawing.Graphics
	{
		public void DrawRectangleBC(Color fgcolor, Color bkColor, Color shadow, Color light, Color border, Rectangle r, int style, int borderwidth)
		{
			int		counter;
			Pen		lefttoppen;
			Pen		rightbottompen;

			switch(style)
			{
				case 0:		//no border
					break;
				case 1:			//three d border
					for (counter = 0;counter < width; counter++)
					{
					    lefttoppen = hpenfill;
					    rightbottompen = hpenfill;
					    if (0 == counter)
						{
					    	lefttoppen = hpenshadow;
					    	rightbottompen = hpenlight;
						}
					    if (1 == counter)
						{
					    	lefttoppen = hpenborder;
					    	rightbottompen = hpenface;
						}
						if (itemaction == ODA_DRAWENTIRE)
					    {
							SelectObject(hdc,lefttoppen);
							MoveToEx(hdc,rect.left,rect.top,NULL);
							LineTo(hdc,rect.right,rect.top);
							MoveToEx(hdc,rect.left,rect.top,NULL);
							LineTo(hdc,rect.left,rect.bottom);

							SelectObject(hdc,rightbottompen);
							MoveToEx(hdc,rect.right-1,rect.top,NULL);
							LineTo(hdc,rect.right-1,rect.bottom);
							MoveToEx(hdc,rect.left,rect.bottom-1,NULL);
							LineTo(hdc,rect.right,rect.bottom-1);
						}
						InflateRect(&rect, -1, -1);
					}
					break;
				case 2:			//single color
				default:
					break;
			}
		}
	}
	*/
	/// <summary>
    ///    Summary description for Control1.
    /// </summary>
    public class Control1 : System.WinForms.UserControl
    {
        /// <summary>
        ///    Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components;
		public enum enumBorderStyle
		{
			No_Border=0,
			ThreeD_Border=1,
			Single_Color_Border=2
		};

		public enum enumBackStyle
		{
			Transparent = 0,
		    Opaque = 1
		};

		public enum enumFlashType
		{
			NoneFlash = 0,
		    BlackFlash = 1,
		    BackgroundFlash = 2,
		    ColorFlash = 3
		};

        public enum enumAlignment
		{
			Left,
			Center,
			Right
		};

		public enum enumVAlignment
		{
			Top,
			Center,
			Bottom
		};
		

		private enumAlignment		m_alignment;
		private enumVAlignment		m_valignment;
		private enumBackStyle		m_backstyle;
		private Color				m_bordercolor;
		private enumBorderStyle		m_borderstyle;
		private int					m_bordersize;
		//private string				m_caption;
		private Color		 		m_flashcolor;
		private enumFlashType		m_flashtype;
		private bool				m_wordwrap;
		private int					m_flashtime;
		//private	short				m_sButton;
		//private short				m_overcontrol;
		//private int					m_lastflash;
		private Color				m_curflash;
		Timer						m_timer;
        private TickTimerHandler	m_eTimer; 

		//properties
		//Alignment
		[
			Category("Appearance"),
			Description("Specifies the horizontal alignment of text."),
			Browsable(true),
			Persistable(PersistableSupport.All),
			DefaultValue(enumAlignment.Left)
		]
		public enumAlignment Alignment 
		{
			get 
			{
				return m_alignment;
			}
			set 
			{
				m_alignment = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
		//VAlignment
		[
			Category("Appearance"),
			Description("Specifies the vertical alignment of text."),
			Browsable(true),
			Persistable(PersistableSupport.All),
			DefaultValue(enumVAlignment.Top)
		]
		public enumVAlignment VAlignment 
		{
			get 
			{
				return m_valignment;
			}
			set 
			{
				m_valignment = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
		//Caption
		[
			Category("Appearance"),
			Description("Specifies the caption."),
		]
		public string Caption 
		{
			get 
			{
				return Text;
			}
			set 
			{
				Text = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        

        [
        Browsable(false),
        Persistable(PersistableSupport.None)
        ]
        public string Text 
		{
            override get 
			{
                return base.Text;
            }
            override set 
			{
                base.Text = value;
            }
			
        }

		//BackStyle
		[
			Category("Appearance"),
			Description("Specifies the tranparancy of the control."),
			Browsable(true),
			Persistable(PersistableSupport.All),
			DefaultValue(enumBackStyle.Opaque)
		]
		public enumBackStyle BackStyle 
		{
			get 
			{
				return m_backstyle;
			}
			set 
			{
				m_backstyle = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //BorderColor
		[
			Category("Appearance"),
			Description("Specifies the color of the border."),
			Browsable(true),
			Persistable(PersistableSupport.All)
		]
		public Color BorderColor 
		{
			get 
			{
				return m_bordercolor;
			}
			set 
			{
				m_bordercolor = value;
				m_curflash = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //BorderStyle
		[
			Category("Appearance"),
			Description("Specifies the style of the border."),
			Browsable(true),
			Persistable(PersistableSupport.All),
			DefaultValue(enumBorderStyle.Single_Color_Border)
		]
		public enumBorderStyle BorderStyle
		{
			get 
			{
				return m_borderstyle;
			}
			set 
			{
				m_borderstyle = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
		
		//BorderSize
		[
			Category("Appearance"),
			Editor(typeof(netLabel.BorderSizeEditor),typeof(UITypeEditor)),
			Description("Specifies the size of the border."),
			Browsable(true),
			Persistable(PersistableSupport.All),
			DefaultValue(1)
		]
		public int BorderSize
		{
			get 
			{
				return m_bordersize;
			}
			set 
			{
				m_bordersize = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //FlashColor
		[
			Category("Appearance"),
			Description("Specifies the color that flashes."),
			Browsable(true),
			Persistable(PersistableSupport.All)
		]
		public Color FlashColor 
		{
			get 
			{
				return m_flashcolor;
			}
			set 
			{
				m_flashcolor = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //FlashType
		[
			Category("Appearance"),
			Description("Specifies the type of flashing."),
			Browsable(true),
			Persistable(PersistableSupport.All),
			DefaultValue(enumFlashType.NoneFlash)
		]
		public enumFlashType FlashType
		{
			get 
			{
				return m_flashtype;
			}
			set 
			{
				m_flashtype = value;
				//Invalidate invokes the OnPaint method
				//Invalidate();
				if (m_flashtype>0)
				{
					if (m_flashtime>0)
					{
						if (GetDesignMode()==false)
						{
							m_timer.Start();
						}
					}
				}
			}
		}
        //MouseIcon
        //MousePointer

		//WordWrap
		[
			Category("Appearance"),
			Description("Determines if the text wraps or not."),
			Browsable(true),
			Persistable(PersistableSupport.All),
			DefaultValue(true)
		]
		public bool WordWrap
		{
			get 
			{
				return m_wordwrap;
			}
			set 
			{
				m_wordwrap = value;
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}
        //FlashTime
		[
			Category("Appearance"),
			Description("Specifies the time between flashes."),
			Browsable(true),
			Persistable(PersistableSupport.All),
			DefaultValue(0)
		]
		public int FlashTime
		{
			get 
			{
				return m_flashtime;
			}
			set 
			{
				if (m_flashtype!=enumFlashType.NoneFlash)
				{
					m_timer.Stop();
				}
				m_flashtime = value;
				if (value>0)
				{
					m_timer.Interval=value;
					if (m_flashtype>0)
					{
						if (GetDesignMode()==false)
						{
							m_timer.Start();
						}
						
					}
				}
				//Invalidate invokes the OnPaint method
				Invalidate();
			}
		}

        //Timer Event
        [Description("Raised when the Timer message hits")]
        public event TickTimerHandler TickTimer {
            get 
			{
                return m_eTimer;
            }
            set 
			{
                m_eTimer = value;
            }
        }
        protected virtual void OnTimer(TickTimerEventArgs e) 
		{
            if (m_eTimer != null) 
				m_eTimer(this, e);
        }


		//BorderSizePix)(/*[out, retval]*/ long *pVal);

		//methods
		//STDMETHOD (About)();
        //STDMETHOD (Refresh)();
		public override void  Refresh()
		{
			Invalidate();
		}
        //STDMETHOD (Zorder)(VARIANT);
		public Control1()
        {
            // This call is required by the WinForms Form Designer.
            InitializeComponent();

            // TODO: Add any initialization after the InitForm call

        }

        protected override void OnPaint(System.WinForms.PaintEventArgs sa)
		{
			StringFormat style = new StringFormat();
			switch (m_alignment)
			{
				case enumAlignment.Left:
					style.Alignment=System.Drawing.StringAlignment.Near;
					break;
				case enumAlignment.Center:
					style.Alignment=System.Drawing.StringAlignment.Center;
					break;
				case enumAlignment.Right:
					style.Alignment=System.Drawing.StringAlignment.Far;
					break;
				default:
					style.Alignment=System.Drawing.StringAlignment.Near;
					break;
			}
			switch (m_valignment)
			{
				case enumVAlignment.Top:
					style.LineAlignment=System.Drawing.StringAlignment.Near;
					break;
				case enumVAlignment.Center:
					style.LineAlignment=System.Drawing.StringAlignment.Center;
					break;
				case enumVAlignment.Bottom:
					style.LineAlignment=System.Drawing.StringAlignment.Far;
					break;
				default:
					style.LineAlignment=System.Drawing.StringAlignment.Near;
					break;
			}

			sa.Graphics.DrawRectangle(new Pen(m_curflash,1),ClientRectangle.Top,ClientRectangle.Left,ClientRectangle.Width,ClientRectangle.Height);
			sa.Graphics.DrawString(Text,Font,new SolidBrush(ForeColor),ClientRectangle,style);

		}
		/// <summary>
        ///    Clean up any resources being used.
        /// </summary>
        public override void Dispose()
        {
            base.Dispose();
            components.Dispose();
        }

        /// <summary>
        ///    Required method for Designer support - do not modify 
        ///    the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
		{
			m_timer = new Timer();
			m_timer.Tick+=new EventHandler(OnTimerEvent);
			this.components = new System.ComponentModel.Container ();
			//@this.TrayLargeIcon = false;
			//@this.TrayAutoArrange = true;
			//@this.TrayHeight = 0;
			this.Size = new System.Drawing.Size (344, 80);
			this.AccessibleName = "netLabel";
//			this.Click += new System.EventHandler (this.Control1_Click);
			this.Alignment = enumAlignment.Left;
			this.VAlignment = enumVAlignment.Top;
			this.BackStyle = enumBackStyle.Opaque;
			this.BorderColor=System.Drawing.SystemColors.WindowFrame;
			this.BorderStyle=enumBorderStyle.Single_Color_Border;
			this.BorderSize=1;
			this.Caption="netLabel";
			this.FlashColor=System.Drawing.Color.Red;
			this.FlashTime = 0;
			this.FlashType = enumFlashType.NoneFlash;
			this.WordWrap = true;
		}

		protected void Control1_Click (object sender, System.EventArgs e)
		{

		}

		private void OnTimerEvent(object source, EventArgs e)
		{
			TickTimerEventArgs	etick = new TickTimerEventArgs(false,m_curflash);
				
			OnTimer(etick);
			if (!etick.Cancel)
			{
					
				if (m_curflash==m_bordercolor)
				{
					m_curflash=m_flashcolor;
				}
				else
				{
					m_curflash=m_bordercolor;
				}
				Invalidate();
			}
		}
    }
}
